"""The eyemask commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - EYEMASK:MASK<x>:COUNt:HITS?
    - EYEMASK:MASK<x>:COUNt:SEG<y>:HITS?
    - EYEMASK:MASK<x>:CREATor?
    - EYEMASK:MASK<x>:ENAbled {ON|OFF}
    - EYEMASK:MASK<x>:ENAbled?
    - EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit?
    - EYEMASK:MASK<x>:MASKfile <QString>
    - EYEMASK:MASK<x>:MASKfile?
    - EYEMASK:MASK<x>:TESt:SAMple:THReshold <NR1>
    - EYEMASK:MASK<x>:TESt:SAMple:THReshold?
    - EYEMASK:MASK<x>:TESt:STATUS?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class EyemaskMaskItemTestStatus(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:TESt:STATUS`` command.

    Description:
        - This query-only command returns the mask hit test status.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:TESt:STATUS?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:TESt:STATUS?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - EYEMASK:MASK<x>:TESt:STATUS?
        ```

    Info:
        - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
    """


class EyemaskMaskItemTestSampleThreshold(SCPICmdWrite, SCPICmdRead):
    """The ``EYEMASK:MASK<x>:TESt:SAMple:THReshold`` command.

    Description:
        - This command sets or queries the total number of hit violations that will cause a mask
          test failure.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:TESt:SAMple:THReshold?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``EYEMASK:MASK<x>:TESt:SAMple:THReshold?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``EYEMASK:MASK<x>:TESt:SAMple:THReshold value`` command.

    SCPI Syntax:
        ```
        - EYEMASK:MASK<x>:TESt:SAMple:THReshold <NR1>
        - EYEMASK:MASK<x>:TESt:SAMple:THReshold?
        ```

    Info:
        - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
        - ``<NR1>`` is a positive integer indicating the number of mask hits required to cause a
          fail condition for that mask test.
    """


class EyemaskMaskItemTestSample(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:TESt:SAMple`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:TESt:SAMple?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:TESt:SAMple?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Info:
        - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).

    Properties:
        - ``.threshold``: The ``EYEMASK:MASK<x>:TESt:SAMple:THReshold`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._threshold = EyemaskMaskItemTestSampleThreshold(
            device, f"{self._cmd_syntax}:THReshold"
        )

    @property
    def threshold(self) -> EyemaskMaskItemTestSampleThreshold:
        """Return the ``EYEMASK:MASK<x>:TESt:SAMple:THReshold`` command.

        Description:
            - This command sets or queries the total number of hit violations that will cause a mask
              test failure.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:TESt:SAMple:THReshold?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``EYEMASK:MASK<x>:TESt:SAMple:THReshold?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``EYEMASK:MASK<x>:TESt:SAMple:THReshold value`` command.

        SCPI Syntax:
            ```
            - EYEMASK:MASK<x>:TESt:SAMple:THReshold <NR1>
            - EYEMASK:MASK<x>:TESt:SAMple:THReshold?
            ```

        Info:
            - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
            - ``<NR1>`` is a positive integer indicating the number of mask hits required to cause a
              fail condition for that mask test.
        """
        return self._threshold


class EyemaskMaskItemTest(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:TESt`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:TESt?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:TESt?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Info:
        - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).

    Properties:
        - ``.sample``: The ``EYEMASK:MASK<x>:TESt:SAMple`` command tree.
        - ``.status``: The ``EYEMASK:MASK<x>:TESt:STATUS`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._sample = EyemaskMaskItemTestSample(device, f"{self._cmd_syntax}:SAMple")
        self._status = EyemaskMaskItemTestStatus(device, f"{self._cmd_syntax}:STATUS")

    @property
    def sample(self) -> EyemaskMaskItemTestSample:
        """Return the ``EYEMASK:MASK<x>:TESt:SAMple`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:TESt:SAMple?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:TESt:SAMple?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Info:
            - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).

        Sub-properties:
            - ``.threshold``: The ``EYEMASK:MASK<x>:TESt:SAMple:THReshold`` command.
        """
        return self._sample

    @property
    def status(self) -> EyemaskMaskItemTestStatus:
        """Return the ``EYEMASK:MASK<x>:TESt:STATUS`` command.

        Description:
            - This query-only command returns the mask hit test status.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:TESt:STATUS?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:TESt:STATUS?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - EYEMASK:MASK<x>:TESt:STATUS?
            ```

        Info:
            - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
        """
        return self._status


class EyemaskMaskItemMaskfile(SCPICmdWrite, SCPICmdRead):
    """The ``EYEMASK:MASK<x>:MASKfile`` command.

    Description:
        - This command sets or queries the current mask definition file name for the specified mask
          test.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:MASKfile?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:MASKfile?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EYEMASK:MASK<x>:MASKfile value``
          command.

    SCPI Syntax:
        ```
        - EYEMASK:MASK<x>:MASKfile <QString>
        - EYEMASK:MASK<x>:MASKfile?
        ```

    Info:
        - ``<x>`` is the number of the specified mask test (or mask test plot?).
        - ``<QString>`` is a quoted string that defines the file path that specifies the locationof
          the mask file to use, in the format '[<path>]<filename.ext>'. Specifying a pathis
          optional. If no path is entered, the instrument will search in the current
          workingdirectory as set in ``FILESystem:CWD``.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EyemaskMaskItemMaskoffsetHorizontalAutofit(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit`` command.

    Description:
        - This command returns the mask offset value in the specified mask in seconds. The mask test
          number is specified by <x>

    Usage:
        - Using the ``.query()`` method will send the
          ``EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit?`` query.
        - Using the ``.verify(value)`` method will send the
          ``EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit?`` query and raise an AssertionError if
          the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit?
        ```
    """


class EyemaskMaskItemMaskoffsetHorizontal(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:MASKOffset:HORizontal`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:MASKOffset:HORizontal?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``EYEMASK:MASK<x>:MASKOffset:HORizontal?`` query and raise an AssertionError if the
          returned value does not match ``value``.

    Properties:
        - ``.autofit``: The ``EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._autofit = EyemaskMaskItemMaskoffsetHorizontalAutofit(
            device, f"{self._cmd_syntax}:AUTOfit"
        )

    @property
    def autofit(self) -> EyemaskMaskItemMaskoffsetHorizontalAutofit:
        """Return the ``EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit`` command.

        Description:
            - This command returns the mask offset value in the specified mask in seconds. The mask
              test number is specified by <x>

        Usage:
            - Using the ``.query()`` method will send the
              ``EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit?`` query.
            - Using the ``.verify(value)`` method will send the
              ``EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit?`` query and raise an AssertionError
              if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit?
            ```
        """
        return self._autofit


class EyemaskMaskItemMaskoffset(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:MASKOffset`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:MASKOffset?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:MASKOffset?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.horizontal``: The ``EYEMASK:MASK<x>:MASKOffset:HORizontal`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._horizontal = EyemaskMaskItemMaskoffsetHorizontal(
            device, f"{self._cmd_syntax}:HORizontal"
        )

    @property
    def horizontal(self) -> EyemaskMaskItemMaskoffsetHorizontal:
        """Return the ``EYEMASK:MASK<x>:MASKOffset:HORizontal`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:MASKOffset:HORizontal?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``EYEMASK:MASK<x>:MASKOffset:HORizontal?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.autofit``: The ``EYEMASK:MASK<x>:MASKOffset:HORizontal:AUTOfit`` command.
        """
        return self._horizontal


class EyemaskMaskItemEnabled(SCPICmdWrite, SCPICmdRead):
    """The ``EYEMASK:MASK<x>:ENAbled`` command.

    Description:
        - This command enables or disables eye mask testing in the specified plot.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:ENAbled?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:ENAbled?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EYEMASK:MASK<x>:ENAbled value``
          command.

    SCPI Syntax:
        ```
        - EYEMASK:MASK<x>:ENAbled {ON|OFF}
        - EYEMASK:MASK<x>:ENAbled?
        ```

    Info:
        - ``MASK<x>`` is the number of the specified mask plot.
        - ``ON`` enables the eye mask test.
        - ``OFF`` disables the eye mask test.
    """


class EyemaskMaskItemCreator(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:CREATor`` command.

    Description:
        - This query-only command returns the name of the eye diagram plot that created the mask.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:CREATor?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:CREATor?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - EYEMASK:MASK<x>:CREATor?
        ```

    Info:
        - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
    """


class EyemaskMaskItemCountSegItemHits(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:COUNt:SEG<y>:HITS`` command.

    Description:
        - This command returns the number of hit violations for the specified segment (area).

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:COUNt:SEG<y>:HITS?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:COUNt:SEG<y>:HITS?``
          query and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - EYEMASK:MASK<x>:COUNt:SEG<y>:HITS?
        ```

    Info:
        - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
        - ``SEG<y>`` is the number of the mask segment for which to return hit violations data.
    """


class EyemaskMaskItemCountSegItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``EYEMASK:MASK<x>:COUNt:SEG<y>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:COUNt:SEG<y>?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:COUNt:SEG<y>?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Info:
        - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
        - ``SEG<y>`` is the number of the mask segment for which to return hit violations data.

    Properties:
        - ``.hits``: The ``EYEMASK:MASK<x>:COUNt:SEG<y>:HITS`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._hits = EyemaskMaskItemCountSegItemHits(device, f"{self._cmd_syntax}:HITS")

    @property
    def hits(self) -> EyemaskMaskItemCountSegItemHits:
        """Return the ``EYEMASK:MASK<x>:COUNt:SEG<y>:HITS`` command.

        Description:
            - This command returns the number of hit violations for the specified segment (area).

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:COUNt:SEG<y>:HITS?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``EYEMASK:MASK<x>:COUNt:SEG<y>:HITS?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        SCPI Syntax:
            ```
            - EYEMASK:MASK<x>:COUNt:SEG<y>:HITS?
            ```

        Info:
            - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
            - ``SEG<y>`` is the number of the mask segment for which to return hit violations data.
        """
        return self._hits


class EyemaskMaskItemCountHits(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:COUNt:HITS`` command.

    Description:
        - This command returns the total number of hit violations for all segments in the specified
          mask test.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:COUNt:HITS?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:COUNt:HITS?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - EYEMASK:MASK<x>:COUNt:HITS?
        ```

    Info:
        - ``MASK<x>`` is the number of the specified mask test in an eye diagram plot.
    """


class EyemaskMaskItemCount(SCPICmdRead):
    """The ``EYEMASK:MASK<x>:COUNt`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:COUNt?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:COUNt?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Info:
        - ``MASK<x>`` is the number of the specified mask test in an eye diagram plot.

    Properties:
        - ``.hits``: The ``EYEMASK:MASK<x>:COUNt:HITS`` command.
        - ``.seg``: The ``EYEMASK:MASK<x>:COUNt:SEG<y>`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._hits = EyemaskMaskItemCountHits(device, f"{self._cmd_syntax}:HITS")
        self._seg: Dict[int, EyemaskMaskItemCountSegItem] = DefaultDictPassKeyToFactory(
            lambda x: EyemaskMaskItemCountSegItem(device, f"{self._cmd_syntax}:SEG{x}")
        )

    @property
    def hits(self) -> EyemaskMaskItemCountHits:
        """Return the ``EYEMASK:MASK<x>:COUNt:HITS`` command.

        Description:
            - This command returns the total number of hit violations for all segments in the
              specified mask test.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:COUNt:HITS?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:COUNt:HITS?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - EYEMASK:MASK<x>:COUNt:HITS?
            ```

        Info:
            - ``MASK<x>`` is the number of the specified mask test in an eye diagram plot.
        """
        return self._hits

    @property
    def seg(self) -> Dict[int, EyemaskMaskItemCountSegItem]:
        """Return the ``EYEMASK:MASK<x>:COUNt:SEG<y>`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:COUNt:SEG<y>?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:COUNt:SEG<y>?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Info:
            - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
            - ``SEG<y>`` is the number of the mask segment for which to return hit violations data.

        Sub-properties:
            - ``.hits``: The ``EYEMASK:MASK<x>:COUNt:SEG<y>:HITS`` command.
        """
        return self._seg


class EyemaskMaskItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``EYEMASK:MASK<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Info:
        - ``MASK<x>`` is the number of the specified mask test in an eye diagram plot.

    Properties:
        - ``.count``: The ``EYEMASK:MASK<x>:COUNt`` command tree.
        - ``.creator``: The ``EYEMASK:MASK<x>:CREATor`` command.
        - ``.enabled``: The ``EYEMASK:MASK<x>:ENAbled`` command.
        - ``.maskoffset``: The ``EYEMASK:MASK<x>:MASKOffset`` command tree.
        - ``.maskfile``: The ``EYEMASK:MASK<x>:MASKfile`` command.
        - ``.test``: The ``EYEMASK:MASK<x>:TESt`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._count = EyemaskMaskItemCount(device, f"{self._cmd_syntax}:COUNt")
        self._creator = EyemaskMaskItemCreator(device, f"{self._cmd_syntax}:CREATor")
        self._enabled = EyemaskMaskItemEnabled(device, f"{self._cmd_syntax}:ENAbled")
        self._maskoffset = EyemaskMaskItemMaskoffset(device, f"{self._cmd_syntax}:MASKOffset")
        self._maskfile = EyemaskMaskItemMaskfile(device, f"{self._cmd_syntax}:MASKfile")
        self._test = EyemaskMaskItemTest(device, f"{self._cmd_syntax}:TESt")

    @property
    def count(self) -> EyemaskMaskItemCount:
        """Return the ``EYEMASK:MASK<x>:COUNt`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:COUNt?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:COUNt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Info:
            - ``MASK<x>`` is the number of the specified mask test in an eye diagram plot.

        Sub-properties:
            - ``.hits``: The ``EYEMASK:MASK<x>:COUNt:HITS`` command.
            - ``.seg``: The ``EYEMASK:MASK<x>:COUNt:SEG<y>`` command tree.
        """
        return self._count

    @property
    def creator(self) -> EyemaskMaskItemCreator:
        """Return the ``EYEMASK:MASK<x>:CREATor`` command.

        Description:
            - This query-only command returns the name of the eye diagram plot that created the
              mask.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:CREATor?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:CREATor?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - EYEMASK:MASK<x>:CREATor?
            ```

        Info:
            - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).
        """
        return self._creator

    @property
    def enabled(self) -> EyemaskMaskItemEnabled:
        """Return the ``EYEMASK:MASK<x>:ENAbled`` command.

        Description:
            - This command enables or disables eye mask testing in the specified plot.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:ENAbled?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:ENAbled?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EYEMASK:MASK<x>:ENAbled value``
              command.

        SCPI Syntax:
            ```
            - EYEMASK:MASK<x>:ENAbled {ON|OFF}
            - EYEMASK:MASK<x>:ENAbled?
            ```

        Info:
            - ``MASK<x>`` is the number of the specified mask plot.
            - ``ON`` enables the eye mask test.
            - ``OFF`` disables the eye mask test.
        """
        return self._enabled

    @property
    def maskoffset(self) -> EyemaskMaskItemMaskoffset:
        """Return the ``EYEMASK:MASK<x>:MASKOffset`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:MASKOffset?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:MASKOffset?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.horizontal``: The ``EYEMASK:MASK<x>:MASKOffset:HORizontal`` command tree.
        """
        return self._maskoffset

    @property
    def maskfile(self) -> EyemaskMaskItemMaskfile:
        """Return the ``EYEMASK:MASK<x>:MASKfile`` command.

        Description:
            - This command sets or queries the current mask definition file name for the specified
              mask test.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:MASKfile?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:MASKfile?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EYEMASK:MASK<x>:MASKfile value``
              command.

        SCPI Syntax:
            ```
            - EYEMASK:MASK<x>:MASKfile <QString>
            - EYEMASK:MASK<x>:MASKfile?
            ```

        Info:
            - ``<x>`` is the number of the specified mask test (or mask test plot?).
            - ``<QString>`` is a quoted string that defines the file path that specifies the
              locationof the mask file to use, in the format '[<path>]<filename.ext>'. Specifying a
              pathis optional. If no path is entered, the instrument will search in the current
              workingdirectory as set in ``FILESystem:CWD``.
        """
        return self._maskfile

    @property
    def test(self) -> EyemaskMaskItemTest:
        """Return the ``EYEMASK:MASK<x>:TESt`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>:TESt?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>:TESt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Info:
            - ``MASK<x>`` is the number of the specified mask test (or mask test plot?).

        Sub-properties:
            - ``.sample``: The ``EYEMASK:MASK<x>:TESt:SAMple`` command tree.
            - ``.status``: The ``EYEMASK:MASK<x>:TESt:STATUS`` command.
        """
        return self._test


class Eyemask(SCPICmdRead):
    """The ``EYEMASK`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``EYEMASK?`` query.
        - Using the ``.verify(value)`` method will send the ``EYEMASK?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.mask``: The ``EYEMASK:MASK<x>`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "EYEMASK") -> None:
        super().__init__(device, cmd_syntax)
        self._mask: Dict[int, EyemaskMaskItem] = DefaultDictPassKeyToFactory(
            lambda x: EyemaskMaskItem(device, f"{self._cmd_syntax}:MASK{x}")
        )

    @property
    def mask(self) -> Dict[int, EyemaskMaskItem]:
        """Return the ``EYEMASK:MASK<x>`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``EYEMASK:MASK<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``EYEMASK:MASK<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Info:
            - ``MASK<x>`` is the number of the specified mask test in an eye diagram plot.

        Sub-properties:
            - ``.count``: The ``EYEMASK:MASK<x>:COUNt`` command tree.
            - ``.creator``: The ``EYEMASK:MASK<x>:CREATor`` command.
            - ``.enabled``: The ``EYEMASK:MASK<x>:ENAbled`` command.
            - ``.maskoffset``: The ``EYEMASK:MASK<x>:MASKOffset`` command tree.
            - ``.maskfile``: The ``EYEMASK:MASK<x>:MASKfile`` command.
            - ``.test``: The ``EYEMASK:MASK<x>:TESt`` command tree.
        """
        return self._mask



# pyright: reportConstantRedefinition=none
"""The timer commands module.

These commands are used in the following models:
MP5103

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - timer.cleartime()
    - timer.measure.t()
    - timer.readtime()
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class TimerMeasure(BaseTSPCmd):
    """The ``timer.measure`` command tree.

    Properties and methods:
        - ``.t()``: The ``timer.measure.t()`` function.
    """

    def t(self) -> str:
        """Run the ``timer.measure.t()`` function.

        Description:
            - This function measures the elapsed time since the timer was last reset.

        TSP Syntax:
            ```
            - timer.measure.t()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.t())"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.t()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Timer(BaseTSPCmd):
    """The ``timer`` command tree.

    Properties and methods:
        - ``.cleartime()``: The ``timer.cleartime()`` function.
        - ``.measure``: The ``timer.measure`` command tree.
        - ``.readtime()``: The ``timer.readtime()`` function.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "timer") -> None:
        super().__init__(device, cmd_syntax)
        self._measure = TimerMeasure(device, f"{self._cmd_syntax}.measure")

    @property
    def measure(self) -> TimerMeasure:
        """Return the ``timer.measure`` command tree.

        Sub-properties and sub-methods:
            - ``.t()``: The ``timer.measure.t()`` function.
        """
        return self._measure

    def cleartime(self) -> str:
        """Run the ``timer.cleartime()`` function.

        Description:
            - This function resets the timer to zero (0) seconds.

        TSP Syntax:
            ```
            - timer.cleartime()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.cleartime())"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.cleartime()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def readtime(self) -> str:
        """Run the ``timer.readtime()`` function.

        Description:
            - This function measures the elapsed time since the timer was last reset.

        TSP Syntax:
            ```
            - timer.readtime()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readtime())"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.readtime()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The source commands module.

These commands are used in the following models:
AWG5200

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SOURce:FREQuency <frequency>
    - SOURce:FREQuency?
    - SOURce:IQIMode {I2X|I4X}
    - SOURce:IQIMode?
    - SOURce:RCCouple {ON|OFF|1|0}
    - SOURce:ROSCillator:MULTiplier <NR1>
    - SOURce:ROSCillator:MULTiplier?
    - SOURce[n]:CASSet:CLEar
    - SOURce[n]:CASSet:SEQuence <sequence_name>, <track_number>[,<component_type>]
    - SOURce[n]:CASSet:TYPE?
    - SOURce[n]:CASSet:WAVeform <wfm_name>[,<component_type>]
    - SOURce[n]:CASSet?
    - SOURce[n]:CFRequency <center_frequency>
    - SOURce[n]:DAC:RESolution {12|13|14|15|16}
    - SOURce[n]:DAC:RESolution?
    - SOURce[n]:DDR {ON|OFF|1|0}
    - SOURce[n]:DDR?
    - SOURce[n]:DMODe {NRZ|MIX|RZ}
    - SOURce[n]:DMODe?
    - SOURce[n]:JUMP:FORCe {FIRSt|CURRent|LAST|END|<NR1>}
    - SOURce[n]:JUMP:PATTern:FORCe <pattern>
    - SOURce[n]:MARKer[m]:DELay <NR3>
    - SOURce[n]:MARKer[m]:DELay?
    - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude <NRf>
    - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude?
    - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH <NRf>
    - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH?
    - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW <NRf>
    - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW?
    - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet <NR3>
    - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet?
    - SOURce[n]:POWer:LEVel:IMMediate:AMPLitude <NRf>
    - SOURce[n]:POWer:LEVel:IMMediate:AMPLitude?
    - SOURce[n]:RMODe {CONTinuous|TRIGgered|TCONtinuous|GATed}
    - SOURce[n]:RMODe?
    - SOURce[n]:SCSTep?
    - SOURce[n]:SKEW <skew>
    - SOURce[n]:SKEW?
    - SOURce[n]:TINPut {ITRigger|ATRigger|BTRigger}
    - SOURce[n]:TINPut?
    - SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude <NRf>
    - SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude?
    - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS <NRf>
    - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle {ON|OFF|1|0}
    - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle?
    - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS?
    - SOURce[n]:VOLTage:LEVel:IMMediate:HIGH <NRf>
    - SOURce[n]:VOLTage:LEVel:IMMediate:HIGH?
    - SOURce[n]:VOLTage:LEVel:IMMediate:LOW <NRf>
    - SOURce[n]:VOLTage:LEVel:IMMediate:LOW?
    - SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet <NRf>
    - SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet?
    - SOURce[n]:WAVeform <wfm_name>
    - SOURce[n]:WAVeform?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    SCPICmdWriteNoArguments,
    ValidatedChannel,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SourceItemWaveform(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:WAVeform`` command.

    Description:
        - This command sets or returns the name of the waveform assigned to the specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:WAVeform?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:WAVeform?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce[n]:WAVeform value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:WAVeform <wfm_name>
        - SOURce[n]:WAVeform?
        ```
    """


class SourceItemVoltageLevelImmediateOffset(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet`` command.

    Description:
        - This command sets or returns the offset for the waveform associated with the specified
          channel. This command is only valid when the output path is set to DC Amplified.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet <NRf>
        - SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet?
        ```
    """


class SourceItemVoltageLevelImmediateLow(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW`` command.

    Description:
        - This command sets or returns the low voltage level for the waveform associated with a
          channel. This command is only valid when the output path is set to Direct.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:VOLTage:LEVel:IMMediate:LOW <NRf>
        - SOURce[n]:VOLTage:LEVel:IMMediate:LOW?
        ```

    Info:
        - ``*RST`` sets this to -250 mV.
    """


class SourceItemVoltageLevelImmediateHigh(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH`` command.

    Description:
        - This command sets or returns the high voltage level for the waveform associated with the
          specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:VOLTage:LEVel:IMMediate:HIGH <NRf>
        - SOURce[n]:VOLTage:LEVel:IMMediate:HIGH?
        ```

    Info:
        - ``*RST`` sets this to 250 mV.
    """


class SourceItemVoltageLevelImmediateBiasEnable(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle`` command.

    Description:
        - This command sets or returns the state (enabled or disabled) of the Bias control. When
          enabled, a bias level can be added to the output. The Output Path must be set to one of
          the AC output paths.

    Usage:
        - Using the ``.query()`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle?`` query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle?`` query and raise an AssertionError if
          the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle {ON|OFF|1|0}
        - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle?
        ```

    Info:
        - ``0`` or OFF disables using an AC bias setting. 1 or ON enables using an AC bias setting.
        - ``*RST`` sets this to 0.
    """


class SourceItemVoltageLevelImmediateBias(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS`` command.

    Description:
        - This command sets or returns the Bias (for AC output paths) for the waveform associated
          with the specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS <NRf>
        - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS?
        ```

    Properties:
        - ``.enable``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._enable = SourceItemVoltageLevelImmediateBiasEnable(
            device, f"{self._cmd_syntax}:ENABle"
        )

    @property
    def enable(self) -> SourceItemVoltageLevelImmediateBiasEnable:
        """Return the ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle`` command.

        Description:
            - This command sets or returns the state (enabled or disabled) of the Bias control. When
              enabled, a bias level can be added to the output. The Output Path must be set to one
              of the AC output paths.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle?`` query and raise an AssertionError
              if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle {ON|OFF|1|0}
            - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle?
            ```

        Info:
            - ``0`` or OFF disables using an AC bias setting. 1 or ON enables using an AC bias
              setting.
            - ``*RST`` sets this to 0.
        """
        return self._enable


class SourceItemVoltageLevelImmediateAmplitude(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude`` command.

    Description:
        - This command sets or returns the amplitude for the waveform associated with a channel in
          units of volts.

    Usage:
        - Using the ``.query()`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude?`` query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude <NRf>
        - SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude?
        ```

    Info:
        - ``pp`` to 355.7 mV pp.
        - ``pp`` to 2 V.
    """


class SourceItemVoltageLevelImmediate(SCPICmdRead):
    """The ``SOURce[n]:VOLTage:LEVel:IMMediate`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage:LEVel:IMMediate?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:VOLTage:LEVel:IMMediate?``
          query and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.bias``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS`` command.
        - ``.high``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH`` command.
        - ``.low``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW`` command.
        - ``.offset``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet`` command.
        - ``.amplitude``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bias = SourceItemVoltageLevelImmediateBias(device, f"{self._cmd_syntax}:BIAS")
        self._high = SourceItemVoltageLevelImmediateHigh(device, f"{self._cmd_syntax}:HIGH")
        self._low = SourceItemVoltageLevelImmediateLow(device, f"{self._cmd_syntax}:LOW")
        self._offset = SourceItemVoltageLevelImmediateOffset(device, f"{self._cmd_syntax}:OFFSet")
        self._amplitude = SourceItemVoltageLevelImmediateAmplitude(
            device, f"{self._cmd_syntax}:AMPLitude"
        )

    @property
    def bias(self) -> SourceItemVoltageLevelImmediateBias:
        """Return the ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS`` command.

        Description:
            - This command sets or returns the Bias (for AC output paths) for the waveform
              associated with the specified channel.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS <NRf>
            - SOURce[n]:VOLTage:LEVel:IMMediate:BIAS?
            ```

        Sub-properties:
            - ``.enable``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS:ENABle`` command.
        """
        return self._bias

    @property
    def high(self) -> SourceItemVoltageLevelImmediateHigh:
        """Return the ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH`` command.

        Description:
            - This command sets or returns the high voltage level for the waveform associated with
              the specified channel.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:VOLTage:LEVel:IMMediate:HIGH <NRf>
            - SOURce[n]:VOLTage:LEVel:IMMediate:HIGH?
            ```

        Info:
            - ``*RST`` sets this to 250 mV.
        """
        return self._high

    @property
    def low(self) -> SourceItemVoltageLevelImmediateLow:
        """Return the ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW`` command.

        Description:
            - This command sets or returns the low voltage level for the waveform associated with a
              channel. This command is only valid when the output path is set to Direct.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:VOLTage:LEVel:IMMediate:LOW <NRf>
            - SOURce[n]:VOLTage:LEVel:IMMediate:LOW?
            ```

        Info:
            - ``*RST`` sets this to -250 mV.
        """
        return self._low

    @property
    def offset(self) -> SourceItemVoltageLevelImmediateOffset:
        """Return the ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet`` command.

        Description:
            - This command sets or returns the offset for the waveform associated with the specified
              channel. This command is only valid when the output path is set to DC Amplified.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet <NRf>
            - SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet?
            ```
        """
        return self._offset

    @property
    def amplitude(self) -> SourceItemVoltageLevelImmediateAmplitude:
        """Return the ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude`` command.

        Description:
            - This command sets or returns the amplitude for the waveform associated with a channel
              in units of volts.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude?`` query and raise an AssertionError if
              the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude <NRf>
            - SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude?
            ```

        Info:
            - ``pp`` to 355.7 mV pp.
            - ``pp`` to 2 V.
        """
        return self._amplitude


class SourceItemVoltageLevel(SCPICmdRead):
    """The ``SOURce[n]:VOLTage:LEVel`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage:LEVel?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:VOLTage:LEVel?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.immediate``: The ``SOURce[n]:VOLTage:LEVel:IMMediate`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._immediate = SourceItemVoltageLevelImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def immediate(self) -> SourceItemVoltageLevelImmediate:
        """Return the ``SOURce[n]:VOLTage:LEVel:IMMediate`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage:LEVel:IMMediate?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:VOLTage:LEVel:IMMediate?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.bias``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:BIAS`` command.
            - ``.high``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:HIGH`` command.
            - ``.low``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:LOW`` command.
            - ``.offset``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:OFFSet`` command.
            - ``.amplitude``: The ``SOURce[n]:VOLTage:LEVel:IMMediate:AMPLitude`` command.
        """
        return self._immediate


class SourceItemVoltage(SCPICmdRead):
    """The ``SOURce[n]:VOLTage`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:VOLTage?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.level``: The ``SOURce[n]:VOLTage:LEVel`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._level = SourceItemVoltageLevel(device, f"{self._cmd_syntax}:LEVel")

    @property
    def level(self) -> SourceItemVoltageLevel:
        """Return the ``SOURce[n]:VOLTage:LEVel`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage:LEVel?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:VOLTage:LEVel?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.immediate``: The ``SOURce[n]:VOLTage:LEVel:IMMediate`` command tree.
        """
        return self._level


class SourceItemTinput(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:TINPut`` command.

    Description:
        - This command sets or returns the trigger input source of the specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:TINPut?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:TINPut?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce[n]:TINPut value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:TINPut {ITRigger|ATRigger|BTRigger}
        - SOURce[n]:TINPut?
        ```

    Info:
        - ``ITRigger`` selects the internal trigger signal as the trigger source. (The A and B Force
          Trigger buttons are not active.) ATRigger selects trigger input A. BTRigger selects
          trigger input B. [n] determines the channel number. If omitted, interpreted as 1.
        - ``*RST`` sets this to ATR.
    """


class SourceItemSkew(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:SKEW`` command.

    Description:
        - This command sets or returns the skew (relative timing of the analog output) for the
          waveform associated with the specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:SKEW?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:SKEW?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce[n]:SKEW value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:SKEW <skew>
        - SOURce[n]:SKEW?
        ```

    Info:
        - ``*RST`` sets this to 0.
    """


class SourceItemScstep(SCPICmdRead):
    """The ``SOURce[n]:SCSTep`` command.

    Description:
        - This command allows you to read the current step of the sequence while the system is
          running.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:SCSTep?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:SCSTep?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SOURce[n]:SCSTep?
        ```
    """


class SourceItemRmode(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:RMODe`` command.

    Description:
        - This command sets or returns the run mode of the specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:RMODe?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:RMODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce[n]:RMODe value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:RMODe {CONTinuous|TRIGgered|TCONtinuous|GATed}
        - SOURce[n]:RMODe?
        ```

    Info:
        - ``CONTinuous`` sets the Run Mode to Continuous (not waiting for trigger). TRIGgered sets
          the Run Mode to Triggered, waiting for a trigger event. One waveform play out cycle
          completes, then play out stops, waiting for the next trigger event. TCONtinuous sets the
          Run Mode to Triggered Continuous, waiting for a trigger. Once a trigger is received, the
          waveform plays out continuously. GATed sets the Run Mode to only playout a waveform while
          the trigger is enabled.
        - ``*RST`` sets this to CONT.
    """


class SourceItemPowerLevelImmediateAmplitude(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude`` command.

    Description:
        - This command sets or returns the amplitude for the waveform associated with the specified
          channel in units of dBm for the filtered path. This command is only valid when the output
          path is set to AC. Option AC is required. If Option AC is not installed, sending the
          command causes an error message.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:POWer:LEVel:IMMediate:AMPLitude <NRf>
        - SOURce[n]:POWer:LEVel:IMMediate:AMPLitude?
        ```
    """


class SourceItemPowerLevelImmediate(SCPICmdRead):
    """The ``SOURce[n]:POWer:LEVel:IMMediate`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:POWer:LEVel:IMMediate?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:POWer:LEVel:IMMediate?``
          query and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.amplitude``: The ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._amplitude = SourceItemPowerLevelImmediateAmplitude(
            device, f"{self._cmd_syntax}:AMPLitude"
        )

    @property
    def amplitude(self) -> SourceItemPowerLevelImmediateAmplitude:
        """Return the ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude`` command.

        Description:
            - This command sets or returns the amplitude for the waveform associated with the
              specified channel in units of dBm for the filtered path. This command is only valid
              when the output path is set to AC. Option AC is required. If Option AC is not
              installed, sending the command causes an error message.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude?`` query and raise an AssertionError if
              the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:POWer:LEVel:IMMediate:AMPLitude <NRf>
            - SOURce[n]:POWer:LEVel:IMMediate:AMPLitude?
            ```
        """
        return self._amplitude


class SourceItemPowerLevel(SCPICmdRead):
    """The ``SOURce[n]:POWer:LEVel`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:POWer:LEVel?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:POWer:LEVel?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.immediate``: The ``SOURce[n]:POWer:LEVel:IMMediate`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._immediate = SourceItemPowerLevelImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def immediate(self) -> SourceItemPowerLevelImmediate:
        """Return the ``SOURce[n]:POWer:LEVel:IMMediate`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:POWer:LEVel:IMMediate?``
              query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:POWer:LEVel:IMMediate?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.amplitude``: The ``SOURce[n]:POWer:LEVel:IMMediate:AMPLitude`` command.
        """
        return self._immediate


class SourceItemPower(SCPICmdRead):
    """The ``SOURce[n]:POWer`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:POWer?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:POWer?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.level``: The ``SOURce[n]:POWer:LEVel`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._level = SourceItemPowerLevel(device, f"{self._cmd_syntax}:LEVel")

    @property
    def level(self) -> SourceItemPowerLevel:
        """Return the ``SOURce[n]:POWer:LEVel`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:POWer:LEVel?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:POWer:LEVel?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.immediate``: The ``SOURce[n]:POWer:LEVel:IMMediate`` command tree.
        """
        return self._level


class SourceItemMarkerItemVoltageLevelImmediateOffset(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet`` command.

    Description:
        - This command sets or returns the offset voltage of the selected marker of the selected
          channel.

    Usage:
        - Using the ``.query()`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet?`` query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet?`` query and raise an AssertionError
          if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet <NR3>
        - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet?
        ```

    Info:
        - ``*RST`` sets this to 500 mV.
    """


class SourceItemMarkerItemVoltageLevelImmediateLow(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW`` command.

    Description:
        - This command sets or returns the marker low voltage level of the specified marker of the
          specified channel.

    Usage:
        - Using the ``.query()`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW?`` query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW?`` query and raise an AssertionError if
          the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW <NRf>
        - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW?
        ```

    Info:
        - ``*RST`` sets this to 0 V.
    """


class SourceItemMarkerItemVoltageLevelImmediateHigh(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH`` command.

    Description:
        - This command sets or returns the marker high voltage level of the specified marker of the
          specified channel.

    Usage:
        - Using the ``.query()`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH?`` query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH?`` query and raise an AssertionError if
          the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH <NRf>
        - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH?
        ```

    Info:
        - ``*RST`` sets all markers to 1 V.
    """


class SourceItemMarkerItemVoltageLevelImmediateAmplitude(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude`` command.

    Description:
        - This command sets or returns the marker voltage amplitude of the specified marker of the
          specified channel.

    Usage:
        - Using the ``.query()`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude?`` query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude <NRf>
        - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude?
        ```

    Info:
        - ``*RST`` sets this to 1 V for all markers.
    """


class SourceItemMarkerItemVoltageLevelImmediate(SCPICmdRead):
    """The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate`` command tree.

    Usage:
        - Using the ``.query()`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate?`` query.
        - Using the ``.verify(value)`` method will send the
          ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate?`` query and raise an AssertionError if the
          returned value does not match ``value``.

    Properties:
        - ``.high``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH`` command.
        - ``.low``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW`` command.
        - ``.offset``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet`` command.
        - ``.amplitude``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._high = SourceItemMarkerItemVoltageLevelImmediateHigh(
            device, f"{self._cmd_syntax}:HIGH"
        )
        self._low = SourceItemMarkerItemVoltageLevelImmediateLow(device, f"{self._cmd_syntax}:LOW")
        self._offset = SourceItemMarkerItemVoltageLevelImmediateOffset(
            device, f"{self._cmd_syntax}:OFFSet"
        )
        self._amplitude = SourceItemMarkerItemVoltageLevelImmediateAmplitude(
            device, f"{self._cmd_syntax}:AMPLitude"
        )

    @property
    def high(self) -> SourceItemMarkerItemVoltageLevelImmediateHigh:
        """Return the ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH`` command.

        Description:
            - This command sets or returns the marker high voltage level of the specified marker of
              the specified channel.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH <NRf>
            - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH?
            ```

        Info:
            - ``*RST`` sets all markers to 1 V.
        """
        return self._high

    @property
    def low(self) -> SourceItemMarkerItemVoltageLevelImmediateLow:
        """Return the ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW`` command.

        Description:
            - This command sets or returns the marker low voltage level of the specified marker of
              the specified channel.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW?`` query and raise an AssertionError
              if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW <NRf>
            - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW?
            ```

        Info:
            - ``*RST`` sets this to 0 V.
        """
        return self._low

    @property
    def offset(self) -> SourceItemMarkerItemVoltageLevelImmediateOffset:
        """Return the ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet`` command.

        Description:
            - This command sets or returns the offset voltage of the selected marker of the selected
              channel.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet <NR3>
            - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet?
            ```

        Info:
            - ``*RST`` sets this to 500 mV.
        """
        return self._offset

    @property
    def amplitude(self) -> SourceItemMarkerItemVoltageLevelImmediateAmplitude:
        """Return the ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude`` command.

        Description:
            - This command sets or returns the marker voltage amplitude of the specified marker of
              the specified channel.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude <NRf>
            - SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude?
            ```

        Info:
            - ``*RST`` sets this to 1 V for all markers.
        """
        return self._amplitude


class SourceItemMarkerItemVoltageLevel(SCPICmdRead):
    """The ``SOURce[n]:MARKer[m]:VOLTage:LEVel`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:MARKer[m]:VOLTage:LEVel?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:MARKer[m]:VOLTage:LEVel?``
          query and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.immediate``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._immediate = SourceItemMarkerItemVoltageLevelImmediate(
            device, f"{self._cmd_syntax}:IMMediate"
        )

    @property
    def immediate(self) -> SourceItemMarkerItemVoltageLevelImmediate:
        """Return the ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate`` command tree.

        Usage:
            - Using the ``.query()`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate?`` query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate?`` query and raise an AssertionError if
              the returned value does not match ``value``.

        Sub-properties:
            - ``.high``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:HIGH`` command.
            - ``.low``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:LOW`` command.
            - ``.offset``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:OFFSet`` command.
            - ``.amplitude``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate:AMPLitude`` command.
        """
        return self._immediate


class SourceItemMarkerItemVoltage(SCPICmdRead):
    """The ``SOURce[n]:MARKer[m]:VOLTage`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:MARKer[m]:VOLTage?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:MARKer[m]:VOLTage?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.level``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._level = SourceItemMarkerItemVoltageLevel(device, f"{self._cmd_syntax}:LEVel")

    @property
    def level(self) -> SourceItemMarkerItemVoltageLevel:
        """Return the ``SOURce[n]:MARKer[m]:VOLTage:LEVel`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:MARKer[m]:VOLTage:LEVel?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``SOURce[n]:MARKer[m]:VOLTage:LEVel?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.immediate``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel:IMMediate`` command tree.
        """
        return self._level


class SourceItemMarkerItemDelay(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:MARKer[m]:DELay`` command.

    Description:
        - This command sets or returns the delay for the specified marker of the specified channel.
          Marker delay is independent for each channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:MARKer[m]:DELay?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:MARKer[m]:DELay?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce[n]:MARKer[m]:DELay value``
          command.

    SCPI Syntax:
        ```
        - SOURce[n]:MARKer[m]:DELay <NR3>
        - SOURce[n]:MARKer[m]:DELay?
        ```

    Info:
        - ``*RST`` sets all channel marker delays to 0.
    """


class SourceItemMarkerItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``SOURce[n]:MARKer[m]`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:MARKer[m]?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:MARKer[m]?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.delay``: The ``SOURce[n]:MARKer[m]:DELay`` command.
        - ``.voltage``: The ``SOURce[n]:MARKer[m]:VOLTage`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._delay = SourceItemMarkerItemDelay(device, f"{self._cmd_syntax}:DELay")
        self._voltage = SourceItemMarkerItemVoltage(device, f"{self._cmd_syntax}:VOLTage")

    @property
    def delay(self) -> SourceItemMarkerItemDelay:
        """Return the ``SOURce[n]:MARKer[m]:DELay`` command.

        Description:
            - This command sets or returns the delay for the specified marker of the specified
              channel. Marker delay is independent for each channel.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:MARKer[m]:DELay?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:MARKer[m]:DELay?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce[n]:MARKer[m]:DELay value``
              command.

        SCPI Syntax:
            ```
            - SOURce[n]:MARKer[m]:DELay <NR3>
            - SOURce[n]:MARKer[m]:DELay?
            ```

        Info:
            - ``*RST`` sets all channel marker delays to 0.
        """
        return self._delay

    @property
    def voltage(self) -> SourceItemMarkerItemVoltage:
        """Return the ``SOURce[n]:MARKer[m]:VOLTage`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:MARKer[m]:VOLTage?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:MARKer[m]:VOLTage?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.level``: The ``SOURce[n]:MARKer[m]:VOLTage:LEVel`` command tree.
        """
        return self._voltage


class SourceItemJumpPatternForce(SCPICmdWrite):
    """The ``SOURce[n]:JUMP:PATTern:FORCe`` command.

    Description:
        - This command generates an event, forcing the sequencer to the step specified by the
          pattern in the pattern jump table. If the sequence is playing on both channels, the force
          jump is applied to both channels simultaneously.

    Usage:
        - Using the ``.write(value)`` method will send the ``SOURce[n]:JUMP:PATTern:FORCe value``
          command.

    SCPI Syntax:
        ```
        - SOURce[n]:JUMP:PATTern:FORCe <pattern>
        ```
    """


class SourceItemJumpPattern(SCPICmdRead):
    """The ``SOURce[n]:JUMP:PATTern`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:JUMP:PATTern?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:JUMP:PATTern?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.force``: The ``SOURce[n]:JUMP:PATTern:FORCe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._force = SourceItemJumpPatternForce(device, f"{self._cmd_syntax}:FORCe")

    @property
    def force(self) -> SourceItemJumpPatternForce:
        """Return the ``SOURce[n]:JUMP:PATTern:FORCe`` command.

        Description:
            - This command generates an event, forcing the sequencer to the step specified by the
              pattern in the pattern jump table. If the sequence is playing on both channels, the
              force jump is applied to both channels simultaneously.

        Usage:
            - Using the ``.write(value)`` method will send the
              ``SOURce[n]:JUMP:PATTern:FORCe value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:JUMP:PATTern:FORCe <pattern>
            ```
        """
        return self._force


class SourceItemJumpForce(SCPICmdWrite):
    """The ``SOURce[n]:JUMP:FORCe`` command.

    Description:
        - This command forces the sequencer to jump to a specific sequence step for the specified
          channel. A force jump does not require a trigger event to execute the jump.

    Usage:
        - Using the ``.write(value)`` method will send the ``SOURce[n]:JUMP:FORCe value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:JUMP:FORCe {FIRSt|CURRent|LAST|END|<NR1>}
        ```
    """


class SourceItemJump(SCPICmdRead):
    """The ``SOURce[n]:JUMP`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:JUMP?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:JUMP?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.force``: The ``SOURce[n]:JUMP:FORCe`` command.
        - ``.pattern``: The ``SOURce[n]:JUMP:PATTern`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._force = SourceItemJumpForce(device, f"{self._cmd_syntax}:FORCe")
        self._pattern = SourceItemJumpPattern(device, f"{self._cmd_syntax}:PATTern")

    @property
    def force(self) -> SourceItemJumpForce:
        """Return the ``SOURce[n]:JUMP:FORCe`` command.

        Description:
            - This command forces the sequencer to jump to a specific sequence step for the
              specified channel. A force jump does not require a trigger event to execute the jump.

        Usage:
            - Using the ``.write(value)`` method will send the ``SOURce[n]:JUMP:FORCe value``
              command.

        SCPI Syntax:
            ```
            - SOURce[n]:JUMP:FORCe {FIRSt|CURRent|LAST|END|<NR1>}
            ```
        """
        return self._force

    @property
    def pattern(self) -> SourceItemJumpPattern:
        """Return the ``SOURce[n]:JUMP:PATTern`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:JUMP:PATTern?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:JUMP:PATTern?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.force``: The ``SOURce[n]:JUMP:PATTern:FORCe`` command.
        """
        return self._pattern


class SourceItemDmode(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:DMODe`` command.

    Description:
        - This command sets or returns DAC output mode type for the specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:DMODe?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:DMODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce[n]:DMODe value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:DMODe {NRZ|MIX|RZ}
        - SOURce[n]:DMODe?
        ```
    """


class SourceItemDdr(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:DDR`` command.

    Description:
        - This command sets or returns the DDR (2x interpolation) state (enabled or disabled) for
          the specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:DDR?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:DDR?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce[n]:DDR value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:DDR {ON|OFF|1|0}
        - SOURce[n]:DDR?
        ```
    """


class SourceItemDacResolution(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce[n]:DAC:RESolution`` command.

    Description:
        - This command sets or returns the DAC resolution.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:DAC:RESolution?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:DAC:RESolution?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce[n]:DAC:RESolution value``
          command.

    SCPI Syntax:
        ```
        - SOURce[n]:DAC:RESolution {12|13|14|15|16}
        - SOURce[n]:DAC:RESolution?
        ```

    Info:
        - ``*RST`` sets this to 16.
    """


class SourceItemDac(SCPICmdRead):
    """The ``SOURce[n]:DAC`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:DAC?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:DAC?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.resolution``: The ``SOURce[n]:DAC:RESolution`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._resolution = SourceItemDacResolution(device, f"{self._cmd_syntax}:RESolution")

    @property
    def resolution(self) -> SourceItemDacResolution:
        """Return the ``SOURce[n]:DAC:RESolution`` command.

        Description:
            - This command sets or returns the DAC resolution.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:DAC:RESolution?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:DAC:RESolution?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce[n]:DAC:RESolution value``
              command.

        SCPI Syntax:
            ```
            - SOURce[n]:DAC:RESolution {12|13|14|15|16}
            - SOURce[n]:DAC:RESolution?
            ```

        Info:
            - ``*RST`` sets this to 16.
        """
        return self._resolution


class SourceItemCfrequency(SCPICmdWrite):
    """The ``SOURce[n]:CFRequency`` command.

    Description:
        - This command sets or returns the center frequency for the IQ waveform associated with the
          specified channel. Option DIGUP (Digital Upconverter) is required.

    Usage:
        - Using the ``.write(value)`` method will send the ``SOURce[n]:CFRequency value`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:CFRequency <center_frequency>
        ```
    """


class SourceItemCassetWaveform(SCPICmdWrite):
    """The ``SOURce[n]:CASSet:WAVeform`` command.

    Description:
        - This command assigns a waveform (from the waveform list) to the specified channel.

    Usage:
        - Using the ``.write(value)`` method will send the ``SOURce[n]:CASSet:WAVeform value``
          command.

    SCPI Syntax:
        ```
        - SOURce[n]:CASSet:WAVeform <wfm_name>[,<component_type>]
        ```
    """


class SourceItemCassetType(SCPICmdRead):
    """The ``SOURce[n]:CASSet:TYPE`` command.

    Description:
        - This command returns the type of the asset (waveform or sequence) assigned to a channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:CASSet:TYPE?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:CASSet:TYPE?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SOURce[n]:CASSet:TYPE?
        ```
    """


class SourceItemCassetSequence(SCPICmdWrite):
    """The ``SOURce[n]:CASSet:SEQuence`` command.

    Description:
        - This command assigns a track of a sequence (from the sequence list) to the specified
          channel.

    Usage:
        - Using the ``.write(value)`` method will send the ``SOURce[n]:CASSet:SEQuence value``
          command.

    SCPI Syntax:
        ```
        - SOURce[n]:CASSet:SEQuence <sequence_name>, <track_number>[,<component_type>]
        ```
    """


class SourceItemCassetClear(SCPICmdWriteNoArguments):
    """The ``SOURce[n]:CASSet:CLEar`` command.

    Description:
        - This command removes the asset (waveform or sequence) from the specified channel.

    Usage:
        - Using the ``.write()`` method will send the ``SOURce[n]:CASSet:CLEar`` command.

    SCPI Syntax:
        ```
        - SOURce[n]:CASSet:CLEar
        ```
    """


class SourceItemCasset(SCPICmdRead):
    """The ``SOURce[n]:CASSet`` command.

    Description:
        - This command returns the asset (waveform or sequence) assigned to the specified channel.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]:CASSet?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]:CASSet?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SOURce[n]:CASSet?
        ```

    Properties:
        - ``.clear``: The ``SOURce[n]:CASSet:CLEar`` command.
        - ``.sequence``: The ``SOURce[n]:CASSet:SEQuence`` command.
        - ``.type``: The ``SOURce[n]:CASSet:TYPE`` command.
        - ``.waveform``: The ``SOURce[n]:CASSet:WAVeform`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._clear = SourceItemCassetClear(device, f"{self._cmd_syntax}:CLEar")
        self._sequence = SourceItemCassetSequence(device, f"{self._cmd_syntax}:SEQuence")
        self._type = SourceItemCassetType(device, f"{self._cmd_syntax}:TYPE")
        self._waveform = SourceItemCassetWaveform(device, f"{self._cmd_syntax}:WAVeform")

    @property
    def clear(self) -> SourceItemCassetClear:
        """Return the ``SOURce[n]:CASSet:CLEar`` command.

        Description:
            - This command removes the asset (waveform or sequence) from the specified channel.

        Usage:
            - Using the ``.write()`` method will send the ``SOURce[n]:CASSet:CLEar`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:CASSet:CLEar
            ```
        """
        return self._clear

    @property
    def sequence(self) -> SourceItemCassetSequence:
        """Return the ``SOURce[n]:CASSet:SEQuence`` command.

        Description:
            - This command assigns a track of a sequence (from the sequence list) to the specified
              channel.

        Usage:
            - Using the ``.write(value)`` method will send the ``SOURce[n]:CASSet:SEQuence value``
              command.

        SCPI Syntax:
            ```
            - SOURce[n]:CASSet:SEQuence <sequence_name>, <track_number>[,<component_type>]
            ```
        """
        return self._sequence

    @property
    def type(self) -> SourceItemCassetType:
        """Return the ``SOURce[n]:CASSet:TYPE`` command.

        Description:
            - This command returns the type of the asset (waveform or sequence) assigned to a
              channel.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:CASSet:TYPE?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:CASSet:TYPE?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SOURce[n]:CASSet:TYPE?
            ```
        """
        return self._type

    @property
    def waveform(self) -> SourceItemCassetWaveform:
        """Return the ``SOURce[n]:CASSet:WAVeform`` command.

        Description:
            - This command assigns a waveform (from the waveform list) to the specified channel.

        Usage:
            - Using the ``.write(value)`` method will send the ``SOURce[n]:CASSet:WAVeform value``
              command.

        SCPI Syntax:
            ```
            - SOURce[n]:CASSet:WAVeform <wfm_name>[,<component_type>]
            ```
        """
        return self._waveform


#  pylint: disable=too-many-instance-attributes
class SourceItem(ValidatedChannel, SCPICmdRead):
    """The ``SOURce[n]`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce[n]?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce[n]?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.casset``: The ``SOURce[n]:CASSet`` command.
        - ``.cfrequency``: The ``SOURce[n]:CFRequency`` command.
        - ``.dac``: The ``SOURce[n]:DAC`` command tree.
        - ``.ddr``: The ``SOURce[n]:DDR`` command.
        - ``.dmode``: The ``SOURce[n]:DMODe`` command.
        - ``.marker``: The ``SOURce[n]:MARKer[m]`` command tree.
        - ``.power``: The ``SOURce[n]:POWer`` command tree.
        - ``.rmode``: The ``SOURce[n]:RMODe`` command.
        - ``.scstep``: The ``SOURce[n]:SCSTep`` command.
        - ``.skew``: The ``SOURce[n]:SKEW`` command.
        - ``.tinput``: The ``SOURce[n]:TINPut`` command.
        - ``.voltage``: The ``SOURce[n]:VOLTage`` command tree.
        - ``.waveform``: The ``SOURce[n]:WAVeform`` command.
        - ``.jump``: The ``SOURce[n]:JUMP`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "SOURce[n]") -> None:
        super().__init__(device, cmd_syntax)
        self._casset = SourceItemCasset(device, f"{self._cmd_syntax}:CASSet")
        self._cfrequency = SourceItemCfrequency(device, f"{self._cmd_syntax}:CFRequency")
        self._dac = SourceItemDac(device, f"{self._cmd_syntax}:DAC")
        self._ddr = SourceItemDdr(device, f"{self._cmd_syntax}:DDR")
        self._dmode = SourceItemDmode(device, f"{self._cmd_syntax}:DMODe")
        self._marker: Dict[int, SourceItemMarkerItem] = DefaultDictPassKeyToFactory(
            lambda x: SourceItemMarkerItem(device, f"{self._cmd_syntax}:MARKer{x}")
        )
        self._power = SourceItemPower(device, f"{self._cmd_syntax}:POWer")
        self._rmode = SourceItemRmode(device, f"{self._cmd_syntax}:RMODe")
        self._scstep = SourceItemScstep(device, f"{self._cmd_syntax}:SCSTep")
        self._skew = SourceItemSkew(device, f"{self._cmd_syntax}:SKEW")
        self._tinput = SourceItemTinput(device, f"{self._cmd_syntax}:TINPut")
        self._voltage = SourceItemVoltage(device, f"{self._cmd_syntax}:VOLTage")
        self._waveform = SourceItemWaveform(device, f"{self._cmd_syntax}:WAVeform")
        self._jump = SourceItemJump(device, f"{self._cmd_syntax}:JUMP")

    @property
    def casset(self) -> SourceItemCasset:
        """Return the ``SOURce[n]:CASSet`` command.

        Description:
            - This command returns the asset (waveform or sequence) assigned to the specified
              channel.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:CASSet?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:CASSet?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SOURce[n]:CASSet?
            ```

        Sub-properties:
            - ``.clear``: The ``SOURce[n]:CASSet:CLEar`` command.
            - ``.sequence``: The ``SOURce[n]:CASSet:SEQuence`` command.
            - ``.type``: The ``SOURce[n]:CASSet:TYPE`` command.
            - ``.waveform``: The ``SOURce[n]:CASSet:WAVeform`` command.
        """
        return self._casset

    @property
    def cfrequency(self) -> SourceItemCfrequency:
        """Return the ``SOURce[n]:CFRequency`` command.

        Description:
            - This command sets or returns the center frequency for the IQ waveform associated with
              the specified channel. Option DIGUP (Digital Upconverter) is required.

        Usage:
            - Using the ``.write(value)`` method will send the ``SOURce[n]:CFRequency value``
              command.

        SCPI Syntax:
            ```
            - SOURce[n]:CFRequency <center_frequency>
            ```
        """
        return self._cfrequency

    @property
    def dac(self) -> SourceItemDac:
        """Return the ``SOURce[n]:DAC`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:DAC?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:DAC?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.resolution``: The ``SOURce[n]:DAC:RESolution`` command.
        """
        return self._dac

    @property
    def ddr(self) -> SourceItemDdr:
        """Return the ``SOURce[n]:DDR`` command.

        Description:
            - This command sets or returns the DDR (2x interpolation) state (enabled or disabled)
              for the specified channel.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:DDR?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:DDR?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce[n]:DDR value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:DDR {ON|OFF|1|0}
            - SOURce[n]:DDR?
            ```
        """
        return self._ddr

    @property
    def dmode(self) -> SourceItemDmode:
        """Return the ``SOURce[n]:DMODe`` command.

        Description:
            - This command sets or returns DAC output mode type for the specified channel.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:DMODe?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:DMODe?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce[n]:DMODe value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:DMODe {NRZ|MIX|RZ}
            - SOURce[n]:DMODe?
            ```
        """
        return self._dmode

    @property
    def marker(self) -> Dict[int, SourceItemMarkerItem]:
        """Return the ``SOURce[n]:MARKer[m]`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:MARKer[m]?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:MARKer[m]?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.delay``: The ``SOURce[n]:MARKer[m]:DELay`` command.
            - ``.voltage``: The ``SOURce[n]:MARKer[m]:VOLTage`` command tree.
        """
        return self._marker

    @property
    def power(self) -> SourceItemPower:
        """Return the ``SOURce[n]:POWer`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:POWer?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:POWer?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.level``: The ``SOURce[n]:POWer:LEVel`` command tree.
        """
        return self._power

    @property
    def rmode(self) -> SourceItemRmode:
        """Return the ``SOURce[n]:RMODe`` command.

        Description:
            - This command sets or returns the run mode of the specified channel.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:RMODe?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:RMODe?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce[n]:RMODe value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:RMODe {CONTinuous|TRIGgered|TCONtinuous|GATed}
            - SOURce[n]:RMODe?
            ```

        Info:
            - ``CONTinuous`` sets the Run Mode to Continuous (not waiting for trigger). TRIGgered
              sets the Run Mode to Triggered, waiting for a trigger event. One waveform play out
              cycle completes, then play out stops, waiting for the next trigger event. TCONtinuous
              sets the Run Mode to Triggered Continuous, waiting for a trigger. Once a trigger is
              received, the waveform plays out continuously. GATed sets the Run Mode to only playout
              a waveform while the trigger is enabled.
            - ``*RST`` sets this to CONT.
        """
        return self._rmode

    @property
    def scstep(self) -> SourceItemScstep:
        """Return the ``SOURce[n]:SCSTep`` command.

        Description:
            - This command allows you to read the current step of the sequence while the system is
              running.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:SCSTep?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:SCSTep?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SOURce[n]:SCSTep?
            ```
        """
        return self._scstep

    @property
    def skew(self) -> SourceItemSkew:
        """Return the ``SOURce[n]:SKEW`` command.

        Description:
            - This command sets or returns the skew (relative timing of the analog output) for the
              waveform associated with the specified channel.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:SKEW?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:SKEW?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce[n]:SKEW value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:SKEW <skew>
            - SOURce[n]:SKEW?
            ```

        Info:
            - ``*RST`` sets this to 0.
        """
        return self._skew

    @property
    def tinput(self) -> SourceItemTinput:
        """Return the ``SOURce[n]:TINPut`` command.

        Description:
            - This command sets or returns the trigger input source of the specified channel.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:TINPut?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:TINPut?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce[n]:TINPut value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:TINPut {ITRigger|ATRigger|BTRigger}
            - SOURce[n]:TINPut?
            ```

        Info:
            - ``ITRigger`` selects the internal trigger signal as the trigger source. (The A and B
              Force Trigger buttons are not active.) ATRigger selects trigger input A. BTRigger
              selects trigger input B. [n] determines the channel number. If omitted, interpreted as
              1.
            - ``*RST`` sets this to ATR.
        """
        return self._tinput

    @property
    def voltage(self) -> SourceItemVoltage:
        """Return the ``SOURce[n]:VOLTage`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:VOLTage?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:VOLTage?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.level``: The ``SOURce[n]:VOLTage:LEVel`` command tree.
        """
        return self._voltage

    @property
    def waveform(self) -> SourceItemWaveform:
        """Return the ``SOURce[n]:WAVeform`` command.

        Description:
            - This command sets or returns the name of the waveform assigned to the specified
              channel.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:WAVeform?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:WAVeform?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce[n]:WAVeform value`` command.

        SCPI Syntax:
            ```
            - SOURce[n]:WAVeform <wfm_name>
            - SOURce[n]:WAVeform?
            ```
        """
        return self._waveform

    @property
    def jump(self) -> SourceItemJump:
        """Return the ``SOURce[n]:JUMP`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce[n]:JUMP?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce[n]:JUMP?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.force``: The ``SOURce[n]:JUMP:FORCe`` command.
            - ``.pattern``: The ``SOURce[n]:JUMP:PATTern`` command tree.
        """
        return self._jump


class SourceRoscillatorMultiplier(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce:ROSCillator:MULTiplier`` command.

    Description:
        - This command sets or returns the multiplier of the external reference signal when the
          external reference is variable.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce:ROSCillator:MULTiplier?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce:ROSCillator:MULTiplier?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce:ROSCillator:MULTiplier value``
          command.

    SCPI Syntax:
        ```
        - SOURce:ROSCillator:MULTiplier <NR1>
        - SOURce:ROSCillator:MULTiplier?
        ```
    """


class SourceRoscillator(SCPICmdRead):
    """The ``SOURce:ROSCillator`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce:ROSCillator?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce:ROSCillator?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.multiplier``: The ``SOURce:ROSCillator:MULTiplier`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._multiplier = SourceRoscillatorMultiplier(device, f"{self._cmd_syntax}:MULTiplier")

    @property
    def multiplier(self) -> SourceRoscillatorMultiplier:
        """Return the ``SOURce:ROSCillator:MULTiplier`` command.

        Description:
            - This command sets or returns the multiplier of the external reference signal when the
              external reference is variable.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce:ROSCillator:MULTiplier?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce:ROSCillator:MULTiplier?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SOURce:ROSCillator:MULTiplier value`` command.

        SCPI Syntax:
            ```
            - SOURce:ROSCillator:MULTiplier <NR1>
            - SOURce:ROSCillator:MULTiplier?
            ```
        """
        return self._multiplier


class SourceRccouple(SCPICmdWrite):
    """The ``SOURce:RCCouple`` command.

    Description:
        - This command sets or returns the Coupled state (enabled or disabled) of the Run Mode
          control. The Run controls consist of the Run Mode and Trigger Input. The set form of the
          command forces all channels to match channel 1. After the initial coupling of the
          settings, changes made to the Run Mode of any channel affects all channels.

    Usage:
        - Using the ``.write(value)`` method will send the ``SOURce:RCCouple value`` command.

    SCPI Syntax:
        ```
        - SOURce:RCCouple {ON|OFF|1|0}
        ```

    Info:
        - ``*RST`` sets this to 0.
    """


class SourceIqimode(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce:IQIMode`` command.

    Description:
        - This command sets or returns the Baseband IQ Interpolation mode. IQ Interpolation mode is
          coupled for all channels.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce:IQIMode?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce:IQIMode?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce:IQIMode value`` command.

    SCPI Syntax:
        ```
        - SOURce:IQIMode {I2X|I4X}
        - SOURce:IQIMode?
        ```
    """


class SourceFrequency(SCPICmdWrite, SCPICmdRead):
    """The ``SOURce:FREQuency`` command.

    Description:
        - This command sets or returns the clock sample rate of the AWG. [``:CW``] and [``:FIXed``]
          are optional to provide legacy support but provide no added functionality.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce:FREQuency?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce:FREQuency?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SOURce:FREQuency value`` command.

    SCPI Syntax:
        ```
        - SOURce:FREQuency <frequency>
        - SOURce:FREQuency?
        ```

    Info:
        - ``*RST`` sets the frequency to 25 GHz.
    """


class Source(SCPICmdRead):
    """The ``SOURce`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SOURce?`` query.
        - Using the ``.verify(value)`` method will send the ``SOURce?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.frequency``: The ``SOURce:FREQuency`` command.
        - ``.iqimode``: The ``SOURce:IQIMode`` command.
        - ``.rccouple``: The ``SOURce:RCCouple`` command.
        - ``.roscillator``: The ``SOURce:ROSCillator`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "SOURce") -> None:
        super().__init__(device, cmd_syntax)
        self._frequency = SourceFrequency(device, f"{self._cmd_syntax}:FREQuency")
        self._iqimode = SourceIqimode(device, f"{self._cmd_syntax}:IQIMode")
        self._rccouple = SourceRccouple(device, f"{self._cmd_syntax}:RCCouple")
        self._roscillator = SourceRoscillator(device, f"{self._cmd_syntax}:ROSCillator")

    @property
    def frequency(self) -> SourceFrequency:
        """Return the ``SOURce:FREQuency`` command.

        Description:
            - This command sets or returns the clock sample rate of the AWG. [``:CW``] and
              [``:FIXed``] are optional to provide legacy support but provide no added
              functionality.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce:FREQuency?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce:FREQuency?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce:FREQuency value`` command.

        SCPI Syntax:
            ```
            - SOURce:FREQuency <frequency>
            - SOURce:FREQuency?
            ```

        Info:
            - ``*RST`` sets the frequency to 25 GHz.
        """
        return self._frequency

    @property
    def iqimode(self) -> SourceIqimode:
        """Return the ``SOURce:IQIMode`` command.

        Description:
            - This command sets or returns the Baseband IQ Interpolation mode. IQ Interpolation mode
              is coupled for all channels.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce:IQIMode?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce:IQIMode?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SOURce:IQIMode value`` command.

        SCPI Syntax:
            ```
            - SOURce:IQIMode {I2X|I4X}
            - SOURce:IQIMode?
            ```
        """
        return self._iqimode

    @property
    def rccouple(self) -> SourceRccouple:
        """Return the ``SOURce:RCCouple`` command.

        Description:
            - This command sets or returns the Coupled state (enabled or disabled) of the Run Mode
              control. The Run controls consist of the Run Mode and Trigger Input. The set form of
              the command forces all channels to match channel 1. After the initial coupling of the
              settings, changes made to the Run Mode of any channel affects all channels.

        Usage:
            - Using the ``.write(value)`` method will send the ``SOURce:RCCouple value`` command.

        SCPI Syntax:
            ```
            - SOURce:RCCouple {ON|OFF|1|0}
            ```

        Info:
            - ``*RST`` sets this to 0.
        """
        return self._rccouple

    @property
    def roscillator(self) -> SourceRoscillator:
        """Return the ``SOURce:ROSCillator`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SOURce:ROSCillator?`` query.
            - Using the ``.verify(value)`` method will send the ``SOURce:ROSCillator?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.multiplier``: The ``SOURce:ROSCillator:MULTiplier`` command.
        """
        return self._roscillator



# pyright: reportConstantRedefinition=none
"""The ptp commands module.

These commands are used in the following models:
SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - ptp.domain
    - ptp.ds.info
    - ptp.enable
    - ptp.portstate
    - ptp.slavepreferred
    - ptp.time()
    - ptp.utcoffset
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class PtpDs(BaseTSPCmd):
    """The ``ptp.ds`` command tree.

    Properties and methods:
        - ``.info``: The ``ptp.ds.info`` function.
    """

    def info(self) -> str:
        """Run the ``ptp.ds.info`` function.

        Description:
            - This function is a read-only string that returns the settings of the different data
              sets (DS) associated with the IEEE-1588 2008 specification.

        TSP Syntax:
            ```
            - ptp.ds.info
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.info())"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.info()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Ptp(BaseTSPCmd):
    """The ``ptp`` command tree.

    Properties and methods:
        - ``.domain``: The ``ptp.domain`` attribute.
        - ``.ds``: The ``ptp.ds`` command tree.
        - ``.enable``: The ``ptp.enable`` attribute.
        - ``.portstate``: The ``ptp.portstate`` attribute.
        - ``.slavepreferred``: The ``ptp.slavepreferred`` attribute.
        - ``.time()``: The ``ptp.time()`` function.
        - ``.utcoffset``: The ``ptp.utcoffset`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "ptp") -> None:
        super().__init__(device, cmd_syntax)
        self._ds = PtpDs(device, f"{self._cmd_syntax}.ds")

    @property
    def domain(self) -> str:
        """Access the ``ptp.domain`` attribute.

        Description:
            - This attribute describes the IEEE Std 1588-2008 precision time protocol (PTP) domain.

        Usage:
            - Accessing this property will send the ``print(ptp.domain)`` query.
            - Setting this property to a value will send the ``ptp.domain = value`` command.

        TSP Syntax:
            ```
            - ptp.domain = value
            - print(ptp.domain)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".domain"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.domain)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.domain`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @domain.setter
    def domain(self, value: Union[str, float]) -> None:
        """Access the ``ptp.domain`` attribute.

        Description:
            - This attribute describes the IEEE Std 1588-2008 precision time protocol (PTP) domain.

        Usage:
            - Accessing this property will send the ``print(ptp.domain)`` query.
            - Setting this property to a value will send the ``ptp.domain = value`` command.

        TSP Syntax:
            ```
            - ptp.domain = value
            - print(ptp.domain)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".domain", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.domain = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.domain`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def ds(self) -> PtpDs:
        """Return the ``ptp.ds`` command tree.

        Sub-properties and sub-methods:
            - ``.info``: The ``ptp.ds.info`` function.
        """
        return self._ds

    @property
    def enable(self) -> str:
        """Access the ``ptp.enable`` attribute.

        Description:
            - This attribute enables or disables the precision time protocol (PTP) described in
              IEEE-1588 on the Series 3700A.

        Usage:
            - Accessing this property will send the ``print(ptp.enable)`` query.
            - Setting this property to a value will send the ``ptp.enable = value`` command.

        TSP Syntax:
            ```
            - ptp.enable = value
            - print(ptp.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".enable"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.enable)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @enable.setter
    def enable(self, value: Union[str, float]) -> None:
        """Access the ``ptp.enable`` attribute.

        Description:
            - This attribute enables or disables the precision time protocol (PTP) described in
              IEEE-1588 on the Series 3700A.

        Usage:
            - Accessing this property will send the ``print(ptp.enable)`` query.
            - Setting this property to a value will send the ``ptp.enable = value`` command.

        TSP Syntax:
            ```
            - ptp.enable = value
            - print(ptp.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".enable", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.enable = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def portstate(self) -> str:
        """Access the ``ptp.portstate`` attribute.

        Description:
            - This attribute is a read-only value that indicates the state of the IEEE-1588
              precision time protocol (PTP).

        Usage:
            - Accessing this property will send the ``print(ptp.portstate)`` query.

        TSP Syntax:
            ```
            - print(ptp.portstate)
            ```

        Info:
            - ``state``, the ptp.INITIALIZING (0)
              ptp.FAULTY (1)
              ptp.DISABLE (2)
              ptp.LISTENING (3)
              ptp.PRE_MASTER (4)
              ptp.MASTER (5)
              ptp.PASSIVE (6)
              ptp.UNCALIBRATED (7)
              ptp.SLAVE (8)
              ptp.UNKNOWN (9).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".portstate"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.portstate)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.portstate`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def slavepreferred(self) -> str:
        """Access the ``ptp.slavepreferred`` attribute.

        Description:
            - This attribute describes whether you prefer to have the instrument be a subordinate
              clock or not.

        Usage:
            - Accessing this property will send the ``print(ptp.slavepreferred)`` query.
            - Setting this property to a value will send the ``ptp.slavepreferred = value`` command.

        TSP Syntax:
            ```
            - ptp.slavepreferred = value
            - print(ptp.slavepreferred)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".slavepreferred"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.slavepreferred)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.slavepreferred`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @slavepreferred.setter
    def slavepreferred(self, value: Union[str, float]) -> None:
        """Access the ``ptp.slavepreferred`` attribute.

        Description:
            - This attribute describes whether you prefer to have the instrument be a subordinate
              clock or not.

        Usage:
            - Accessing this property will send the ``print(ptp.slavepreferred)`` query.
            - Setting this property to a value will send the ``ptp.slavepreferred = value`` command.

        TSP Syntax:
            ```
            - ptp.slavepreferred = value
            - print(ptp.slavepreferred)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".slavepreferred", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.slavepreferred = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.slavepreferred`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def utcoffset(self) -> str:
        """Access the ``ptp.utcoffset`` attribute.

        Description:
            - This attribute describes the offset, in seconds, between UTC and PTP.

        Usage:
            - Accessing this property will send the ``print(ptp.utcoffset)`` query.
            - Setting this property to a value will send the ``ptp.utcoffset = value`` command.

        TSP Syntax:
            ```
            - ptp.utcoffset = value
            - print(ptp.utcoffset)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".utcoffset"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.utcoffset)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.utcoffset`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @utcoffset.setter
    def utcoffset(self, value: Union[str, float]) -> None:
        """Access the ``ptp.utcoffset`` attribute.

        Description:
            - This attribute describes the offset, in seconds, between UTC and PTP.

        Usage:
            - Accessing this property will send the ``print(ptp.utcoffset)`` query.
            - Setting this property to a value will send the ``ptp.utcoffset = value`` command.

        TSP Syntax:
            ```
            - ptp.utcoffset = value
            - print(ptp.utcoffset)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".utcoffset", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.utcoffset = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.utcoffset`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def time(self) -> str:
        """Run the ``ptp.time()`` function.

        Description:
            - This function is a read-only string that returns the PTP time in seconds and
              fractional seconds.

        TSP Syntax:
            ```
            - ptp.time()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.time())"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.time()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error
